indexing
	description: "Resource Manager"
	author: "Dr. Thomas Tensi"
	date: "$Date$"
	revision: "$Revision$"

class
	RESOURCE_MANAGER

inherit
	WEL_WINDOWS_ROUTINES

	RESOURCE_CONSTANTS


create
	make


feature {NONE} -- Initialization

	make is
			-- Initialize `Current'
		local
			preferences : GAME_PREFERENCES
		do
			create preferences.make
			set_language(preferences.language)
		end


feature -- Basic Operations

	adapt_id (cd_resource_id : INTEGER) : INTEGER is
			-- returns adapted identification
		do
			result := cd_resource_id + base_index
		end

	-- --------------------

	get_string (resource_id : INTEGER) : STRING is
			-- reads string from resource file
		local
			st : STRING
		do
			st := resource_string_id(resource_id)

			if st = void then
				Result := void
			else
				Result := st.to_string_8
			end
		end

	-- --------------------

	get_cd_string (cd_resource_id : INTEGER) : STRING is
			-- reads culture dependent string from resource file
		do
			result := get_string(adapt_id(cd_resource_id))
		end

	-- --------------------

	expand1 (template : STRING;  param1 : STRING) : STRING is
		-- replaces all occurences of "$1" in `template' by `param1'
		local
			st : STRING
		do
			create st.make_from_string(template)
			st.replace_substring_all("$1", param1)
			Result := st
		end

	-- --------------------

	expand2 (template : STRING;  param1, param2 : STRING) : STRING is
		-- replaces all occurences of "$1" in `template' by `param1'
		-- and "$2" by `param2'
		local
			st : STRING
		do
			st := expand1(template, param1)
			st.replace_substring_all("$2", param2)
			Result := st
		end

	-- --------------------

	expand3 (template : STRING;  param1, param2, param3 : STRING) : STRING is
		-- replaces all occurences of "$1" in `template' by `param1',
		-- "$2" by `param2' and "$3" by `param3'
		local
			st : STRING
		do
			st := expand2(template, param1, param2)
			st.replace_substring_all("$3", param3)
			Result := st
		end

	-- --------------------

	expand4 (template : STRING;  param1, param2, param3, param4 : STRING) : STRING is
		-- replaces all occurences of "$1" in `template' by `param1',
		-- "$2" by `param2', "$3" by `param3' and "$4" by `param4'
		local
			st : STRING
		do
			st := expand3(template, param1, param2, param3)
			st.replace_substring_all("$4", param4)
			Result := st
		end

	-- --------------------

	language_list : LIST[STRING] is
			-- return list of all available user interface languages
		local
			field_list : LIST[STRING]
			row : STRING
		once
			create {ARRAYED_LIST[STRING]} result.make(internal_language_list.count)

			from internal_language_list.start until internal_language_list.after
			loop
				row := internal_language_list.item
				field_list := row.split(',')
				result.extend(field_list[1])
				internal_language_list.forth
			end
		end

	-- --------------------

	set_language (language : STRING) is
			-- set user interface language to `language'
		local
			field_list : LIST[STRING]
			number_string : STRING
			row : STRING
		do
			from internal_language_list.start until internal_language_list.after
			loop
				row := internal_language_list.item
				field_list := row.split(',')

				if language.is_equal(field_list[1]) then
					number_string := field_list[2]
					base_index := number_string.to_integer
				end

				internal_language_list.forth
			end
		end


feature {NONE} -- Implementation

	internal_language_list : LIST[STRING] is
			-- return list of all available user interface languages
		local
			st : STRING
			list : LIST[STRING]
		once
			st := get_string(rsc_id_language_list)
			list := st.split('/')
			result := list
		end


	base_index : INTEGER
		-- base index for language specific resources for current
		-- user interface language

end
