indexing
	description	: "Root class for this WEL Application"
	author		: "Generated by the New WEL Application Wizard"
	date		: "$Date: 2008/9/7 10:23:35 $"
	revision	: "1.0.0"

class
	ROOT_CLASS

inherit
	WEL_APPLICATION
		redefine
			init_application
		end

	WEL_MB_CONSTANTS

	RESOURCE_CONSTANTS
		export
			{NONE} all
		end

create
	make

feature

	main_window: MAIN_WINDOW is
			-- creates the application's main window
		once
			create result.make(current)
		end

	-- --------------------

	init_application is
			-- loads the common controls DLL
		do
			create common_controls_dll.make
			create resource_manager.make
			create sound_manager.make
			create preferences.make
			set_state(State_initializing)
		end


feature -- Access
	game : GAME
	high_score_list : HIGH_SCORE_TABLE
	presentation_state : INTEGER

	-- global visible manager
	resource_manager : RESOURCE_MANAGER
	sound_manager : SOUND_MANAGER

	-- score
	number_of_right_guesses     : INTEGER
	number_of_wrong_guesses     : INTEGER
	number_of_unsolvable_values : INTEGER
	score : INTEGER


feature -- Basic operations
	State_initializing        : INTEGER is  0
	State_before_game         : INTEGER is  1
	State_loading_game        : INTEGER is  2
	State_posting_value       : INTEGER is  3
	State_thinking            : INTEGER is  4
	State_timeout             : INTEGER is  5
	State_showing_evaluation  : INTEGER is  6
	State_showing_solutions   : INTEGER is  7
	State_advancing_value     : INTEGER is  8
	State_showing_score       : INTEGER is  9
	State_showing_preferences : INTEGER is 10

	-- --------------------

	set_state (new_state : INTEGER) is
			-- changes current presentation state to `new_state'
		local
			assumed_unsolvable : BOOLEAN
			is_correct         : BOOLEAN
			old_state          : INTEGER
		do
			old_state := presentation_state
			presentation_state := new_state

			inspect new_state

				when State_initializing then
					sound_manager.set_active(preferences.sound_is_on)
					show_intro_dialog
					set_state(State_before_game)

				when State_before_game then
					game := Void
					main_window.set_game(game)
					main_window.stop_timer
					main_window.clear_timer_field
					main_window.set_unsolvable_button_visibility(
								preferences.unsolvable_button_is_shown)

				when State_loading_game then
					create game.make(preferences.zeroes_are_allowed,
									 preferences.board_size)

					number_of_right_guesses     := 0
					number_of_wrong_guesses     := 0
					number_of_unsolvable_values := 0
					score := 0

					main_window.set_game(game)
					game.start
					set_state(State_posting_value)

				when State_posting_value then
					main_window.reset_board_marking
					main_window.update_value_field
					main_window.start_timer(preferences.think_time)
					presentation_state := State_thinking

				when State_timeout then
					main_window.stop_timer
					main_window.clear_timer_field

					if game.board.get_solving_positions(game.item).count = 0 then
						number_of_unsolvable_values := number_of_unsolvable_values + 1
					else
						number_of_wrong_guesses := number_of_wrong_guesses + 1
						sound_manager.play_sound(sound_manager.SoundKind_bad)
					end

					main_window.update_score(number_of_right_guesses,
					                         number_of_wrong_guesses,
					                         number_of_unsolvable_values,
					                         score)
					set_state(State_showing_solutions)

				when State_showing_evaluation then
					main_window.stop_timer
					assumed_unsolvable := (last_clicked_position = void)

					if game = Void or else game.after then
						is_correct := false
  					elseif game.board.get_solving_positions(game.item).count = 0 then
  						-- no correct positions
  						is_correct := assumed_unsolvable
					elseif assumed_unsolvable then
						is_correct := false
					else
					   	is_correct :=
					   		game.board.position_is_solution(last_clicked_position,
					   										game.item)
					end

					score := (score + score_increment(is_correct)).max(0)

					if is_correct then
						number_of_right_guesses := number_of_right_guesses + 1
						sound_manager.play_sound(sound_manager.SoundKind_okay)
					else
						number_of_wrong_guesses := number_of_wrong_guesses + 1
						sound_manager.play_sound(sound_manager.SoundKind_bad)
					end

					main_window.update_score(number_of_right_guesses,
					                         number_of_wrong_guesses,
					                         number_of_unsolvable_values,
					                         score)

					if assumed_unsolvable then
						set_state(State_showing_solutions)
					else
						main_window.show_evaluation(last_clicked_position, is_correct)
						last_clicked_position := Void
					end

				when State_showing_solutions then
					main_window.clear_timer_field
								main_window.show_solutions

				when State_advancing_value then
					game.forth

					if game.after then
						set_state(State_showing_score)
					else
						set_state(State_posting_value)
					end

				when State_showing_score then
					show_game_result_window
					create high_score_list.make
					high_score_list.put(score)
					show_high_score_window
					set_state(State_before_game)

				when State_showing_preferences then
					-- only allow preference setting in state 'before game'
					if old_state /= State_before_game then
						presentation_state := old_state
					else
						create preferences_window.make(current, main_window, preferences)
						preferences_window.activate
						set_state(State_before_game)
					end
				else

			end
		end

	-- --------------------

	show_game_result_window is
			-- show game result
		local
			body : STRING
			f : FORMAT_INTEGER
			message_box : WEL_MSG_BOX
			title : STRING
		do
			create f.make(8)
			title := resource_manager.get_cd_string(rsc_cd_string_resultdlg_title)
			body := resource_manager.get_cd_string(rsc_cd_string_resultdlg_body)
			body := resource_manager.expand4(body,
			                                 f.formatted(number_of_right_guesses),
			                                 f.formatted(number_of_wrong_guesses),
			                                 f.formatted(number_of_unsolvable_values),
			                                 f.formatted(score))

			create message_box.make
			message_box.basic_message_box(main_window, body, title, mb_ok)
		end

	-- --------------------

	show_high_score_window is
			-- show the high score window for some time and then discard it
		do
			create high_score_window.make(current, main_window)
			high_score_window.show
		end


	-- --------------------

	show_instructions_window is
			-- give some help about the TRIO game
		local
			body : STRING
			message_box : WEL_MSG_BOX
			title : STRING
		do
			title := resource_manager.get_cd_string(rsc_cd_string_instructionsdlg_title)
			body := resource_manager.get_cd_string(rsc_cd_string_instructionsdlg_body)
			create message_box.make
			message_box.basic_message_box(main_window, body, title, mb_ok)
		end


	-- --------------------

	show_intro_dialog is
			-- show intro dialog
		local
			intro_window : SPLASH_WINDOW
		do
			create intro_window.make(main_window, resource_manager)
			intro_window.activate
		end


	-- --------------------

	show_quit_game_query_window : BOOLEAN is
			-- ask user whether game should be ended
		local
			body : STRING
			message_box : WEL_MSG_BOX
			title : STRING
			button : INTEGER
		do
			title := resource_manager.get_cd_string(rsc_cd_string_endgamedlg_title)
			body := resource_manager.get_cd_string(rsc_cd_string_endgamedlg_body)
			create message_box.make
			message_box.question_message_box(main_window, body, title)
			button := message_box.message_box_result
			result := (button = 6)
		end

feature -- Event Handling (public)

	notify_about_click (is_right_click : BOOLEAN;  position : BOARD_POSITION) is
			-- callback routine for main_window to tell about a
			-- mouse click at `position'; the type of click is
			-- specified by `is_right_click'
		local
			is_left_click : BOOLEAN
		do
			is_left_click := not is_right_click

			if presentation_state = State_thinking and is_left_click then
				last_clicked_position := position
				set_state(State_showing_evaluation)
			elseif presentation_state = State_showing_evaluation and is_right_click then
				set_state(State_showing_solutions)
			elseif presentation_state = State_showing_solutions and is_right_click then
				set_state(State_advancing_value)
			end
		end


feature {NONE} -- Implementation
	common_controls_dll: WEL_COMMON_CONTROLS_DLL
	preferences : GAME_PREFERENCES

	last_clicked_position : BOARD_POSITION

    -- dialog windows
    preferences_window : PREFERENCES_WINDOW
	high_score_window : HIGH_SCORE_WINDOW

	-- --------------------

	score_increment (is_correct : BOOLEAN) : INTEGER is
			-- increment on score based on whether solution is correct
			-- or not (`is_correct') and other parameters (like think
			-- time used)
		local
			solution_count : INTEGER
			delta : INTEGER
		do
			if not is_correct then
				Result := -100
			else
				solution_count := game.board.get_solving_positions(game.item).count
				solution_count := solution_count.min(20)
				delta := main_window.remaining_think_time * (21 - solution_count)

				if preferences.think_time > 25 then
					delta := delta // 2
				end

				Result := delta
			end
		end


end -- class ROOT_CLASS
